c     ==============================================================
c                       PROTOSTELLAR DISK MODEL
c           MODULE: ADDITIONAL ROUTINES FOR DUST OPACITIES
c
c     ==============================================================
#include "main.h"



c     --------------------------------------------------------------
c          SOLVE TEMPERATURE OF DUST GRAIN FOR GIVEN MEAN INT
c     --------------------------------------------------------------
      function solve_dust_temperature(meanint,isize,ispec)
      implicit none
      doubleprecision solve_dust_temperature
      doubleprecision dust_integral_emisabs,zbrent
      integer isize,ispec
      doubleprecision meanint(FRSIZE_FREQ)
      external dust_integral_emisabs
c
#include "common_grid.h"
#include "common_dust.h"
c
      common/dustintegral/di_meanint
      doubleprecision di_meanint(FRSIZE_FREQ)
      common/idustintegral/di_isize,di_ispec
      integer di_isize,di_ispec
c
      doubleprecision dum1,dum2
      integer inu
c
c     Put the ispec, isize and meanint into the common
c
      di_isize   = isize
      di_ispec   = ispec
      do inu=1,freq_nr
          di_meanint(inu) = meanint(inu) 
      enddo
c
c     Now check if there is indeed sufficient radiation for a
c     solution.
c
      dum1 = dust_integral_emisabs(0.02d0)
      dum2 = dust_integral_emisabs(10000.d0)
      if(dum1.gt.0.d0) then
c          write(*,*) 'Error: dust temperature < 0.01 K'
c          write(*,*) 'The mean intensities are:'
c          write(*,*) (meanint(inu),inu=1,freq_nr)
c          stop 13
          solve_dust_temperature=0.d0
          return
      endif
      if(dum2.lt.0.d0) then
          write(*,*) 'Error: dust temperature > 10000 K'
          stop 13
      endif
c
c     Now solve for the dust temperature
c
      solve_dust_temperature = zbrent(dust_integral_emisabs,
     %                              2.d-2,1.d4,1.d-1) 
c
      return
      end



c     --------------------------------------------------------------
c                    INTEGRAL OF EMISSION-ABSORPTION
c     --------------------------------------------------------------
      function dust_integral_emisabs(temp)
      implicit none
      doubleprecision dust_integral_emisabs,temp
c
      common/dustintegral/di_meanint
      doubleprecision di_meanint(FRSIZE_FREQ)
      common/idustintegral/di_isize,di_ispec
      integer di_isize,di_ispec
c
#include "common_grid.h"
#include "common_dust.h"
c     
      integer inu
      doubleprecision dummy,kappa,bplanck,find_dust_kappa
c
      dummy = 0.d0
      do inu=1,freq_nr
          kappa = find_dust_kappa(inu,di_isize,di_ispec,temp,-1,0)
          dummy = dummy + kappa * dust_freq_wgt(inu)
     %      * ( bplanck(temp,freq_nu(inu)) - di_meanint(inu) )
      enddo
c
      dust_integral_emisabs = dummy
      return
      end





c     --------------------------------------------------------------
c              FIND TOTAL OPACITY FROM ALL DUST SPECIES
c     --------------------------------------------------------------
      function find_total_dust_opacity(dustrho,dusttemp,inu,iabs,iscat)
      implicit none
      doubleprecision find_total_dust_opacity,find_dust_kappa
      doubleprecision dustrho(DUST_SPECIES_MAX)
      doubleprecision dusttemp(DUST_SIZE_MAX,DUST_SPECIES_MAX)
      integer inu,iabs,iscat
c
#     include "common_grid.h"
#     include "common_dust.h"
c
      doubleprecision rho,temp
      integer ispec,isize
c
      find_total_dust_opacity = 0.d0
c
      do ispec=1,dust_nr_species
          do isize=1,dust_nr_size(ispec)
              rho  = dustrho(ispec)
              temp = dusttemp(isize,ispec)
              find_total_dust_opacity = find_total_dust_opacity +
     %             rho * find_dust_kappa(inu,isize,ispec,temp,
     %             iabs,iscat)
          enddo
      enddo
c     
      return
      end



c     --------------------------------------------------------------
c              FIND TOTAL OPACITY FROM ALL DUST SPECIES
c     --------------------------------------------------------------
      function find_total_dust_emission(dustrho,dusttemp,inu,iabs,iscat)
      implicit none
      doubleprecision find_total_dust_emission,find_dust_kappa,bplanck 
      doubleprecision dustrho(DUST_SPECIES_MAX)
      doubleprecision dusttemp(DUST_SIZE_MAX,DUST_SPECIES_MAX)
      integer inu,iabs,iscat
c
#     include "common_grid.h"
#     include "common_dust.h"
c
      doubleprecision rho,temp
      integer ispec,isize
c
      find_total_dust_emission = 0.d0
c
      do ispec=1,dust_nr_species
          do isize=1,dust_nr_size(ispec)
              rho  = dustrho(ispec)
              temp = dusttemp(isize,ispec)
              find_total_dust_emission = find_total_dust_emission +
     %             rho * find_dust_kappa(inu,isize,ispec,temp,
     %             iabs,iscat) * bplanck(temp,freq_nu(inu))
          enddo
      enddo
c     
      return
      end


