@cgplus.pro
@cgplus.pro
@readopac.pro
@makecgimage.pro
@natconst.pro
;
;===========================================================================
;                        Define the problem 
;===========================================================================
;
rin    = 1.d0 * AU      ; inner radius 
rout   = 3.d2 * AU	; outer radius 
sigma0 = 1.d1           ; dust surface density at 1 AU
alpha  = 0.00d0         ; viscosity coeficient
rstar  = 2.d0 * RS      ; radius of the star
tstar  = 10000.0d0      ; temperature of the star
mstar  = 2.d0 * MS      ; mass of the star
lstar  = 4*!dpi*rstar^2*ss*tstar^4
iradrt = 4              ; 0=real radial rt, 1=flareindex rt, 2=flare+iter
ialg   = 12             ; Use VEF method
plsig  = -1.0           ; powerlaw index surface density
coverz = 9.d0           ; How many scale heights for zmax?
nr     = 30		; number of points in radius
nz     = 70		; number of points in theta in disk
intord = 1              ; Order of integration for vertical transfer
itervrt= 100            ; Nr of iterations for vertical transfer
convcr = 1.d-4          ; Convergence criterion for vertical transfer
nmu    = 40             ; Nr of mu points
mu0    = 1.d-2          ; Minimal mu
incl   = 45.            ; Inclination
;
;
;===========================================================================
;      Make the CG model, which serves as the initial guess for the 
;                  full vertical structure calculation
;===========================================================================
;
o=readopac(nr='1')
;
; Make the setup
;
cg=cgdisk(nr=nr,mstar=mstar/MS,lstar=lstar/LS,tstar=tstar,rin=rin/AU,$
          rout=rout/AU,sigma0=sigma0,plsig=plsig,starvis=2,opac=o)
scg=sed(cg,incl=incl+1d-40)
gastodust=100.
icg=makecgimage(cg,incl=incl+1d-40)
;
; Plot the SED
;
plotsed,scg
;
; Write spectrum
;
nf=n_elements(scg.nu)
openw,1,'spectrum_cg.dat'
printf,1,nf
for inu=0,nf-1 do begin
   printf,1,scg.nu[inu],(scg.lnu_surf[inu]+scg.lnu_mid[inu])/(4*pi*pc^2)
endfor
close,1
;
; Write the image
;
openw,1,'image_cg.dat'
printf,1,nf,nr
printf,1,' '
for inu=0,nf-1 do printf,1,scg.nu[inu]
printf,1,' '
for ir=0,nr-1 do printf,1,cg.r[ir]
printf,1,' '
for inu=0,nf-1 do begin
   for ir=0,nr-1 do begin
      printf,1,icg[inu,ir]
   endfor
endfor
;
end
