@cgplus.pro
@cgplus.pro
@readopac.pro
@problem.pro
@makealbedo.pro
@natconst.pro
;
;===========================================================================
;
; Define the problem 
;
nr     = 40		; number of points in radius
nz     = 70		; number of points in theta in disk
rin    = 5.d-1 * AU     ; inner radius 
rout   = 50 * AU	; outer radius 
coverz = 9.d0           ; How many scale heights for zmax?
sigd0  = 5.d-3          ; dust surface density at 1 AU
plsig  = -0.5           ; powerlaw index surface density
alpha  = 0.00d0         ; viscosity coeficient
rstar  = 2.4 * RS       ; radius of the star
tstar  = 10000.0d0      ; temperature of the star
mstar  = 2.4 * MS       ; mass of the star
lstar  = 4*!dpi*rstar^2*ss*tstar^4
gasdst = 100.           ; gas to dust ratio
iradrt = 4              ; 0=real radial rt, 1=flareindex rt, 2=flare+iter
ialg   = 12             ; Use VEF method
intord = 1              ; Order of integration for vertical transfer
itervrt= 100            ; Nr of iterations for vertical transfer
convcr = 1.d-4          ; Convergence criterion for vertical transfer
nmu    = 40             ; Nr of mu points
mu0    = 1.d-2          ; Minimal mu
incl   = 45.            ; Inclination
idump  = 0              ; Dump every idump iterations
init   = 0              ; If present, read diskstruct_2.dat as initial guess
nrphot = 100            ; Nr of photons for each frequency bin to be followed
iflare = 3              ; Nr of iterations for flaring index
inice  = 1              ; Nice the jobs?
rim    = 0              ; Rim included?
;
;===========================================================================
;             FIRST SOME ELEMENTARY CALCULATIONS BEFOREHAND
;===========================================================================
;
lstar  = 4*!dpi*rstar^2*ss*tstar^4
o      = readopac(nr='noscat')
nf     = n_elements(o.freq)
ivis   = 0
fvis   = 1d4 * cc / 0.55d0 
for inu=1,nf-1 do if((o.freq[inu]-fvis)*   $
       (o.freq[inu-1]-fvis) lt 0.d0) then ivis=inu
eps    = (fvis-o.freq[ivis-1]) / (o.freq[ivis]-o.freq[ivis-1])
kappa  = (1.d0-eps)*o.cabs[ivis-1]+eps*o.cabs[ivis]
;
;===========================================================================
;      Make the CG model, which serves as the initial guess for the 
;                  full vertical structure calculation
;===========================================================================
;
makealbedo,0.d0
oo=readgrainopac('1')
on=readgrainopac('noscat')
;
; The new CGPLUS model with Ts correction (without scattering):
; (Also dump the image of this model!)
;
cg=cgdisk(nr=nr,mstar=mstar/MS,lstar=lstar/LS,tstar=tstar,rin=rin/AU,$
          rout=rout/AU,sigma0=sigd0,plsig=plsig,starvis=2,opac=on)
scg=sed(cg,incl=incl,/nostar,/noscat,image_surf=image_surf,$
                   image_mid=image_mid)
mdisk=gasdst*2*!dpi*integrate(cg.r,cg.r*cg.sigma)
print,'Mass of the disk = ',mdisk/MS,' Msun'
openw,1,'spectrum_cg.dat'
printf,1,nf
printf,1,' '
for inu=0,nf-1 do $
   printf,1,o.freq[inu],scg.lnu[inu]/(4*!dpi*pc^2)
close,1
openw,1,'cgplus.dat'
printf,1,nr
printf,1,' '
for ir=0,nr-1 do printf,1,cg.r[ir],cg.hp[ir],cg.hs[ir],cg.ti[ir],cg.ts[ir]
close,1
openw,1,'image_cgnoscat.dat'
printf,1,nf,nr
printf,1,' '
for inu=0,nf-1 do printf,1,o.freq[inu]
printf,1,' '
for ir=0,nr-1 do printf,1,cg.r[ir]
printf,1,' '
for inu=0,nf-1 do begin
   for ir=0,nr-1 do begin
      printf,1,max([image_mid[inu,ir]+image_surf[inu,ir],1d-99])
   endfor
   printf,1,' '
endfor
close,1
;
;===========================================================================
;     Write some general information to file
;===========================================================================
;
; Make starinfo.inp
;
openw,1,'starinfo.inp'
printf,1,'1'
printf,1,rstar
printf,1,mstar
printf,1,tstar
close,1
;
;
;===========================================================================
;     Make the vertical structure setup based on the CG model above
;===========================================================================
;
a=problem(nz,coverz,cg=cg,nlevr=nlevr,nspanr=nspanr,nstepr=nstepr)
;
; Write the stuff to the input files
;
write_problem_rz,a,iir=iir
;
; Write the Sigma
;
if n_elements(iir) eq 0 then begin
   openw,1,'sigma.inp'
   printf,1,nr
   printf,1,' '
   for ir=1,nr do begin
      printf,1,a.sigma[ir-1],format='(E13.6)'
   endfor
   close,1
endif else begin
   openw,1,'sigma.inp'
   printf,1,1
   printf,1,' '
   printf,1,a.sigma[iir],format='(E13.6)'
   close,1
endelse
;
; The radmc.inp
;
openw,1,'radmonte.inp'
printf,1,1       ; Type of algorithm
printf,1,nrphot  ; Nr of photons
printf,1,100     ; Max nr of scatterings
printf,1,1.d-9   ; Energy limit
printf,1,65286   ; Seed
close,1
;
; The sed.inp
;
openw,1,'sed.inp'
printf,1,'0'
printf,1,'0'    ; Do not include the star in the spectrum
printf,1,incl
printf,1,'0'
printf,1,'0'
close,1
;
; (for radical) Make star spectrum
;
pc=3.08572d18
nf=n_elements(o.cabs)
openw,1,'starspectrum.inp'
printf,1,nf
printf,1,' '
for inu=0,nf-1 do begin
   fnu=(rstar/pc)^2*!dpi*bplanck(o.freq[inu],tstar)
   printf,1,o.freq[inu],fnu
endfor
close,1
; 
; Make flareindex file
;
flare=cg.flareindex
flare[0:3]=flare[4]
if n_elements(iir) eq 0 then begin
   if rim eq 0 then begin
      openw,1,'flareindex.inp'
      printf,1,nr
      for ir=0,nr-1 do begin
         printf,1,flare[ir],format='(E13.6)'
      endfor
      close,1
   endif else begin
      openw,1,'flareindex.inp'
      printf,1,nr
      printf,1,-1d10,format='(E13.6)'
      for ir=1,nr-1 do begin
         printf,1,flare[ir],format='(E13.6)'
      endfor
      close,1
   endelse
endif else begin
   openw,1,'flareindex.inp'
   printf,1,1
   printf,1,flare[iir],format='(E13.6)'
   close,1
endelse
;
; Make the disk.inp file
;
openw,1,'disk.inp'
printf,1,1
printf,1,20
printf,1,0.01
printf,1,coverz
printf,1,alpha
printf,1,iradrt
printf,1,ialg
printf,1,inice
printf,1,iflare
close,1
;
; Write the fullvrt file
;
openw,1,'fullvrt.inp'
printf,1,1
printf,1,1
printf,1,ialg
printf,1,intord
printf,1,itervrt
printf,1,convcr
printf,1,nmu
printf,1,mu0
printf,1,idump
printf,1,init
close,1
;
;
;===========================================================================
;                Now do the run, using both methods
;===========================================================================
;
; First the run without scattering
;
spawn,'cp -f dustopac_noscat.inp dustopac_1.inp'
spawn,'nice ../disk/disk'
spawn,'nice ../sed/sed'
spawn,'cp -f diskstruct_2.dat diskstruct_no.dat'
spawn,'cp -f flareindex.inp flareindex_no.inp'
spawn,'mv -f fullvrt.dat fullvrt_no.dat'
spawn,'mv -f spectrum.dat spectrum_no.dat'
spawn,'mv -f image.dat image_noscat.dat'
;
; Make the same SED, but now with BB rim
;
openw,1,'sed.inp'
printf,1,'0'
printf,1,'0'    ; Do not include the star in the spectrum
printf,1,incl
printf,1,'0'
printf,1,'1'
close,1
spawn,'nice ../sed/sed'
spawn,'mv -f spectrum.dat spectrum_rim_no.dat'
;
end
