;------------------------------------------------------------------------
;                            IMAGE 
;------------------------------------------------------------------------
function makecgimage,a,incl=incl
;
; Constants
;
pi    = !dpi
ss    = 5.6703d-5
AU    = 1.496d13
GG    = 6.672d-8
Msun  = 1.99d33
Lsun  = 3.86d33
mugas = 2.3d0
mp    = 1.6726d-24
kk    = 1.3807d-16
cc    = 2.9979d10
hh    = 6.6262d-27
;
; Prepare the image
;
nr=n_elements(a.r)
nf=n_elements(a.opac.freq)
image = dblarr(nf,nr)
;
if a.id eq 'accrdisk' then begin
   print,'Image does not work for accretion disk'
   stop 
endif
;
if keyword_set(incl) then begin 
   incrad = 0.01745329252*incl
   cosi   = cos(incrad)
endif else begin
   cosi   = 1.d0/sqrt(3.d0)
endelse
;
; Copy some info
;
r=a.r
nr=n_elements(r)
;
; Which disk are we dealing with?
;
cg=0
dw=0
ac=0
id=a.id
irmincg = 0
irmaxcg = 0
if id eq 'cgdisk' then begin
   cg      = 1
   irmincg = 0
   irmaxcg = a.irmaxcg
endif
if id eq 'diffwall' then begin
   print,'Image for diffwall not active'
   stop
endif
if id eq 'combi' then begin
   cg      = 1
   dw      = 1
   irmincg = a.cgstart
   irmaxcg = a.irmaxcg
endif
if not keyword_set(incl) then begin
   incl=0.d0
endif
if cg eq 0 or irmincg ge irmaxcg then begin
   stop
endif
;
; Now make the images
;
nr   = n_elements(a.r)
;;
;; Loop over shells for interior
;;
for ir=irmincg,irmaxcg do begin
   dil         = 1.d0 - exp(-a.sigma[ir]*a.opac.cabs/cosi)
   image[*,ir] = image[*,ir] + dil * bplanck(a.opac.freq,a.Ti[ir])
endfor
;;
;; Loop over shells for surface layer
;;
for ir=irmincg,irmaxcg do begin
   transpf     = 1.d0 + exp(-a.sigma[ir]*a.opac.cabs/cosi)
   sigsurf     = a.tausurf[ir] / kappaplanck(a.opac,a.Ts[ir])
   image[*,ir] = image[*,ir] + sigsurf * a.opac.cabs * $
                 bplanck(a.opac.freq,a.Ts[ir]) * transpf
endfor
;
; Return
;
return,image
end
